class UnexpectedArgument(Exception):
    pass


class UndefinedRequirement(Exception):
    pass


class MultipleInserter(Exception):
    pass


class RepeatedInserter(Exception):
    pass


class PropagationCancelled(Exception):
    pass
